<?php

/**
 * AJAX responder for Packing Slips HTML which can be printed for fulfillment. This will get all orders currently in
 * 'processing' status (and only those for the current user if Sellers are in use, build up HTML for packing slips,
 * and output the html file as a download as response.
 */
add_action( 'wp_ajax_bt_packing_slips', 'bt_packing_slips' );
function bt_packing_slips() {
  // verify request
  check_ajax_referer( 'bt_ecomm_nonce', 'nonce' );

  // run for sellers and admins
  if ( wc_current_user_has_role( 'shop_manager' )
    || wc_current_user_has_role( 'administrator' )
    || wc_current_user_has_role( 'marketplace_administrator' )
  ) {

    // query for all NEMAA's orders since
    $args = [
      'post_type' => 'shop_order',
      'posts_per_page' => -1,
      'post_status' => [
        'wc-processing',
      ],
      'orderby' => 'date',
      'order' => 'ASC',
    ];
    // for sellers themselves, only get their orders
    if (
      bt_use_sellers()
      && wc_current_user_has_role( 'shop_manager' )
      && ! wc_current_user_has_role( 'administrator' )
      && ! wc_current_user_has_role( 'marketplace_administrator' )
    ) {
      $args['author'] = get_current_user_id();
    }
    $orders = new \WP_Query( $args );
    //error_log( __METHOD__ . ' -- orders = ' . print_r( $orders, true ) );

    // build html for each packing slip
    $site_name = get_option( 'blogname' );
    $logo_src = bt_ecomm_option( 'packing_slip_logo_img_src' );
    $vy_logo_src = bt_ecomm_plugin_url() . 'img/venyou-logo-mark.svg';
    if ( empty( $logo_src ) ) {
      $logo_src = bt_ecomm_option( 'placeholder_img_src' );
    }
    $html = '';
    $total_orders = 0;
    $total_items = 0;
    $all_items = [];

    // loop orders
    foreach ( $orders->posts as $order ) {
      $total_orders++;

      $order = wc_get_order( $order->ID );

      // billing info
      $billing_address = $order->get_billing_address_1();
      if ( ! empty( $order->get_billing_address_2() ) ) {
        $billing_address .= '<br>' . $order->get_billing_address_2();
      }
      $billing_phone = clean_phone( $order->get_billing_phone() );

      // shipping info
      $shipping_address = $order->get_shipping_address_1();
      if ( ! empty( $order->get_shipping_address_2() ) ) {
        $shipping_address .= '<br>' . $order->get_shipping_address_2();
      }
      $shipping_phone = clean_phone( $order->get_shipping_phone() );

      // order notes
      if ( $order_notes = $order->get_customer_note() ) {
        $order_notes = '<div class="order-notes">' . nl2br( $order_notes ) . '</div>';
      }

      $html .= <<<HTML
<div class="slip">
    <hr class="no-print" />
    <img class="logo" src="$logo_src" alt="$site_name">
    <h2>
        Thank you for your order!
    </h2>
    <p>
        Order #{$order->get_id()}
    </p>

    <div class="addresses">
        <div class="billing">
            <h6>
                Bill To
            </h6>
            <p class="customer">
                <strong>{$order->get_billing_first_name()} {$order->get_billing_last_name()}</strong><br>
                $billing_address<br>
                {$order->get_billing_city()}, {$order->get_billing_state()} {$order->get_billing_postcode()}
            </p>
            <p>
                {$order->get_billing_email()}<br>
                $billing_phone
            </p>
        </div>
        <div class="shipping">
            <h6>
                Ship To
            </h6>
            <p class="customer">
                <strong>{$order->get_shipping_first_name()} {$order->get_shipping_last_name()}</strong><br>
                $shipping_address<br>
                {$order->get_shipping_city()}, {$order->get_shipping_state()} {$order->get_shipping_postcode()}
            </p>
            <p>
                $shipping_phone
            </p>
        </div>
    </div>    

    $order_notes

    <h6>
        Items
    </h6>
    <table>
        <thead>
            <th colspan="2">Item</th>
            <th>Packed</th>
        </thead>
        <tbody>
HTML;

      foreach ( $order->get_items() as $item ) {
        $total_items += $item->get_quantity();

        // track for all items
        if ( array_key_exists( $item->get_name(), $all_items ) ) {
          // add to existing total
          $all_items[$item->get_name()] += $item->get_quantity();
        } else {
          // init with this total
          $all_items[$item->get_name()] = $item->get_quantity();
        }

        $item_image_src = get_the_post_thumbnail_url( $item->get_product_id(), 'thumbnail' );
        $checkboxes = '';
        $count = $item->get_quantity();
        while ( $count > 0 ) {
          $checkboxes .= '<span></span>';
          $count--;
        }

        $html .= <<<HTML
<tr>
    <td><img src="$item_image_src" alt=""></td>
    <td>{$item->get_quantity()} &times; {$item->get_name()}</td>
    <td>
        <div class="checkboxes">
            $checkboxes
        </div>
    </td>
</tr>
HTML;
      }
      $html .= <<<HTML
        </tbody>
    </table>
    <footer>
        <div>
            Order Placed {$order->get_date_created()->date( 'D, M d Y g:ia' )}
        </div>
        <div class="powered-by">
            <span>$site_name is powered by</span>
            <img src="$vy_logo_src" alt="">
            <span>VenYou Marketplace<span>
        </div>
    </footer>
</div>
HTML;
    }

    // generate picklist html
    $picklist = '';
    ksort( $all_items );
    foreach ( $all_items as $item_name => $qty ) {
      $picklist .= <<<HTML
<div class="picklist">
    <div class="checkboxes">
        <span></span>
    </div>
    <p>
        $qty &times; $item_name
    </p>
</div>
HTML;

    }

    // output html as result
    if ( ! $logo_width = bt_ecomm_option( 'packing_slip_logo_img_width' ) ) {
      $logo_width = '10rem';
    }
    $font_family = "-apple-system, system-ui, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif";
    $timestamp = wp_date( 'Y-m-d g:ia' );
    echo <<<HTML
<!doctype html>
<html lang="en-US">
    <head>
        <title>$site_name &mdash; Packing Slips</title>
        <style>
            * {
                box-sizing: border-box;
            }
            body {
                margin: 1rem;
                font-size: 0.875rem;
                font-family: $font_family;
                color: #111;
            }
            .banner {
                display: flex;
                align-items: center;
                margin-bottom: 2rem;
            }
            .title > * {
                margin: 0.5rem 0 0.5rem 1rem;
            }
            img.logo {
                width: $logo_width;
                height: auto;
            }
            .slip {
                break-before: page;
            }
            .slip header {
                display: flex;
                align-items: center;
            }
            .slip h6 {
                margin: 1rem 0;
                color: #666;
                font-size: 0.75rem;
                text-transform: uppercase;
                letter-spacing: 1px;
            }
            .slip table img {
                width: 6rem;
                height: auto;
            }
            .addresses {
                display: flex;
                gap: 2rem;
            }
            .addresses > * {
                width: calc(50% - 1rem);
            }
            .order-notes {
                margin-bottom: 2rem;
                padding: 0.5rem 0.75rem;
                border: 2px solid #ccc;
            }
            .order-notes p:first-child {
                margin-top: 0;
            }
            .order-notes p:last-child {
                margin-bottom: 0;
            }
            .checkboxes {
                display: flex;
                gap: 0.5rem;
            }
            .checkboxes span {
                width: 1.5rem;
                height: 1.5rem;
                border: 2px solid #999;
            }
            table th,
            table td {
                padding: 0.25rem 1rem;
            }
            table th {
                padding-bottom: 0.5rem;
                text-align: left;
            }
            table td:first-child {
                padding-right: 0;
            }
            hr {
                margin: 2rem 0;
                border: 0;
                border-top: 1px dashed #666;
            }
            .picklist {
                display: flex;
                align-items: center;
                gap: 0.5rem;
                margin-bottom: 0.5rem;
            }
            .picklist p {
                margin: 0;
                font-size: 1rem;
            }
            footer {
                display: flex;
                align-items: center;
                justify-content: space-between;
                margin-top: 2rem;
                font-size: 0.75rem;
            }
            .powered-by {
                display: flex;
                align-items: center;
                gap: 0.25rem;
            }
            .powered-by img {
                width: 2rem;
            }
            @media print {
                .no-print,
                .no-print * {
                    display: none !important;
                }
            }
        </style>
    </head>
    <body>
        <div class="banner">
            <img class="logo" src="$logo_src" alt="$site_name">
            <div class="title">
                <h1>Packing Slips</h1>
            </div>
        </div>
        <p>Generated $timestamp</p>
        <ul>
            <li>Total Orders: $total_orders</li>
            <li>Total Items: $total_items</li>
        </ul>
        <p>Pick List:</p>
        $picklist
        $html
    </body>
</html>
HTML;

  } else {
    echo '<h2>Access denied.</h2>';
  }

  wp_die();
}
