<?php

// force subscriptions to consider it a staging environment if NOT prod or unknown env
if ( ! defined( 'BT_ENV' ) || BT_ENV != 'prod' ) {
	add_filter( 'woocommerce_subscriptions_is_duplicate_site', '__return_true', 9988 );
}


/**
 * @param int $user_id
 * @param int $product_id
 * @param \WC_Order|null $order
 * @param string $note
 * @param string $end_date
 * @param string $period
 *
 * @return false|WC_Subscription|WP_Error
 * @throws WC_Data_Exception
 * @see https://gist.github.com/tripflex/a3123052f36daf18f7cb05391d752223
 */
function bt_create_subscription( $user_id, $product_id, $order = null, $note = '', $end_date = '', $period = '' ) {

	// make sure all required functions and classes exist
	if( ! function_exists( 'wc_create_order' ) || ! function_exists( 'wcs_create_subscription' ) || ! class_exists( 'WC_Subscriptions_Product' ) ){
		return false;
	}

	// get order object that goes with sub
	$use_existing_order = false;
	//error_log( __METHOD__ . ' -- GOING TO USE OR CREATE ORDER! order given = ' . print_r( $order, true ) );
	if ( is_numeric( $order ) ) {
		// ID given; make object
		//error_log( __METHOD__ . ' -- using provided order (id) ' . $order . '...' );
		$order = wc_get_order( $order );
		$use_existing_order = true;

	} else if ( ! is_object( $order ) ) {
		// make a new order since one wasn't provided
		//error_log( __METHOD__ . ' -- creating new order...' );
		$order = wc_create_order( array( 'customer_id' => $user_id ) );

	} else {
		//error_log( __METHOD__ . ' -- using provided order (object) ' . $order->get_id() . '...' );
		$use_existing_order = true;
	}

	if ( is_wp_error( $order ) ) {
		error_log( __METHOD__ . ' -- order error = ' . print_r( $order, true ) );
		\App\bt_email_alert(
			'Error Creating Subscription [' . get_bloginfo('name') . ']',
			'Order error: ' . $order->get_error_message()
		);
		return false;
	}

	// set up product data
	$product = wc_get_product( $product_id );

	if ( is_wp_error( $product ) ) {
		error_log( __METHOD__ . ' -- product error = ' . print_r( $product, true ) );
		\App\bt_email_alert(
			'Error Creating Subscription [' . get_bloginfo('name') . ']',
			'Product error: ' . $product->get_error_message()
		);
		return false;
	}

	// set up user data
	$user = get_user_by( 'ID', $user_id );

	$fname     = $user->first_name;
	$lname     = $user->last_name;
	$email     = $user->user_email;
	if ( ! $use_existing_order ) {
		$address = array(
			'first_name' => $fname,
			'last_name'  => $lname,
			'email'      => $email,
			'address_1'  => get_user_meta( $user_id, 'billing_address_1', true ),
			'address_2'  => get_user_meta( $user_id, 'billing_address_2', true ),
			'city'       => get_user_meta( $user_id, 'billing_city', true ),
			'state'      => get_user_meta( $user_id, 'billing_postcode', true ),
			'postcode'   => get_user_meta( $user_id, 'billing_country', true ),
			'country'    => get_user_meta( $user_id, 'billing_state', true ),
		);

		// add addresses and product to new order
		$order->set_address( $address, 'billing' );
		$order->set_address( $address, 'shipping' );
		$order->add_product( $product, 1 );
	}


	// create subscription
	$sub_args = [
		'order_id' => $order->get_id(),
		'status' => 'pending', // Status should be initially set to pending to match how normal checkout process goes
		'billing_period' => ( $period ? $period : \WC_Subscriptions_Product::get_period( $product ) ),
		'billing_interval' => \WC_Subscriptions_Product::get_interval( $product )
	];
	//error_log( __METHOD__ . ' -- sub_args = ' . print_r( $sub_args, true ) );
	$sub = wcs_create_subscription( $sub_args );

	if( is_wp_error( $sub ) ){
		error_log( __METHOD__ . ' -- subscription error = ' . print_r( $sub, true ) );
		\App\bt_email_alert(
			'Error Creating Subscription [' . get_bloginfo('name') . ']',
			'Subscription error: ' . $sub->get_error_message()
		);
		return false;
	}

	// Modeled after WC_Subscriptions_Cart::calculate_subscription_totals()
	$start_date = gmdate( 'Y-m-d H:i:s' );
	// Add product to subscription
	$sub->add_product( $product, 1 );

	$dates = [
		'trial_end'    => \WC_Subscriptions_Product::get_trial_expiration_date( $product, $start_date ),
		'next_payment' => \WC_Subscriptions_Product::get_first_renewal_payment_date( $product, $start_date ),
		'end'          => ( $end_date ? $end_date : \WC_Subscriptions_Product::get_expiration_date( $product, $start_date )),
	];
	//error_log( __METHOD__ . ' -- dates = ' . print_r( $dates, true ) );

	$sub->update_dates( $dates );
	$sub->calculate_totals();

	// determine if payment is pending (mail in check)
	$pending_payment = false;
	if ( $order->get_payment_method() == 'cheque' ) {
		$pending_payment = true;
	}

	$note = ! empty( $note ) ? $note : __( 'Programmatically generated.' );
	// Update order status with custom note
	if ( ! $use_existing_order ) {
		$order->update_status( 'completed', $note, true );
	}
	// Also update subscription status to active from pending (and add note)
	$sub->update_status( ( $pending_payment ? 'on-hold' : 'active' ), $note, true );

	return $sub;
}


/**
 * Given a subscription ID, return the status of its renewal order.
 */
function bt_get_subscription_renewal_order_status( $sub_id ) {
	global $wpdb;

	$sub_id = intval( $sub_id );

	$sql = "
                    select p.`post_status`
                    from `wp_posts` p
                    join `wp_postmeta` m on m.`post_id` = p.`ID`
                    where
                        m.`meta_value` = $sub_id
                        and m.`meta_key` = '_subscription_renewal'
                ";
	return( $wpdb->get_var( $sql ) );

}